/****************************************************************

    VocProc
    Copyright (C) 2010 Igor Brkić <igor@hyperglitch.com>

    some code taken from Faust DSP: http://faust.grame.fr/

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*****************************************************************/



#include <QDomDocument>
#include <QFile>

// some of these could very probably be removed
#include <stdlib.h>
#include <stdio.h>
#include <limits.h>
#include <math.h>
#include <time.h>
#include <unistd.h>
#include <fcntl.h>
#include <pwd.h>
#include <iostream>
#include <fstream>
#include <list>

#include "vproc_dsp.h"
#include "vproc_jack.h"

using namespace std;

// number of input and output channels
int gNumInChans;
int gNumOutChans;

// Jack ports
jack_port_t *input_ports[256];
jack_port_t *output_ports[256];

// tables of noninterleaved input and output channels for FAUST
float* gInChannel[256];
float* gOutChannel[256];

mydsp	DSP;
list<UI*> UI::fGuiList;

int main( int argc, char *argv[] )
{

    UI*             interface = new QTGUI(argc, argv);
    jack_client_t*  client;
    char            jackname[256];
    char            rcfilename[256];
    char**          physicalInPorts;
    char**          physicalOutPorts;

    float **scales;
    int scalesNum;

    // load scales from XML file

    QFile file("scales.xml");
    if(!file.open( QIODevice::ReadOnly)){
        cout<<"File scales.xml not fount in current directory.\n";
        return -1;
    }
    QDomDocument doc("ScalesML");
    QString err;
    if(!doc.setContent(&file, &err)){
        cout<<"XML file error...\n";
        cout<<err.toStdString();
        file.close();
        return -2;
    }
    file.close();
    QDomElement root=doc.documentElement();
    if(root.tagName()!="scales"){
        cout<<"Invalid XML file.\n";
        return -3;
    }
    QDomNode n=root.firstChild();
    int counter=0;
    scales=(float**)malloc(sizeof(float*));
    while(!n.isNull()){
        QDomElement e = n.toElement();
        if( !e.isNull() ){
            if( e.tagName() == "scale" ){
                counter++;
                cout << "loaded \"" << e.attribute("name", "").toStdString() << "\"...\n";

                scales[counter-1]=(float*)malloc(100*sizeof(float));    // scale can have max 100 notes :)
                int length=0;
                QDomNode tmp = e.firstChild();
                while(!tmp.isNull()){
                    // read notes
                    if(!tmp.isComment()){
                        scales[counter-1][length+1]=tmp.toElement().text().toFloat();
                        length++;
                    }
                    tmp=tmp.nextSibling();
                }
                scales[counter-1][0]=length;
                scales=(float**)realloc(scales, (counter+1)*sizeof(float*));
            }
        }
        n = n.nextSibling();
    }
    scalesNum=counter;



    srand((unsigned)time(NULL));
    snprintf(jackname, 255, "%s-%d", basename(argv[0]), (int)((float)rand()/RAND_MAX*1000));

    char* home = getenv("HOME");
    snprintf(rcfilename, 255, "%s/.%src", home, basename(argv[0]));

    //if ((client = jack_client_new(jackname)) == 0) {
    if ((client = jack_client_open(jackname, JackNullOption, NULL)) == 0) {
        fprintf(stderr, "jack server not running?\n");
        return 1;
    }

    jack_set_process_callback(client, process, 0);
    jack_set_sample_rate_callback(client, srate, 0);
    jack_on_shutdown(client, jack_shutdown, 0);

    gNumInChans = DSP.getNumInputs();
    gNumOutChans = DSP.getNumOutputs();

    char buf[256];
    snprintf(buf, 256, "voice");
    input_ports[0] = jack_port_register(client, buf, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
    snprintf(buf, 256, "carrier");
    input_ports[1] = jack_port_register(client, buf, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);

    for (int i = 0; i < gNumOutChans; i++) {
        char buf[256];
        snprintf(buf, 256, "out_%d", i);
        output_ports[i] = jack_port_register(client, buf, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
    }

    DSP.init(jack_get_sample_rate(client), scales, scalesNum);
    DSP.buildUserInterface(interface);

    interface->recallState(rcfilename);

    physicalInPorts = (char **)jack_get_ports(client, NULL, NULL, JackPortIsPhysical|JackPortIsInput);
    physicalOutPorts = (char **)jack_get_ports(client, NULL, NULL, JackPortIsPhysical|JackPortIsOutput);

    if (jack_activate(client)) {
        fprintf(stderr, "cannot activate client");
        return 1;
    }

    if (physicalOutPorts != NULL) {
        for (int i = 0; i < gNumInChans && physicalOutPorts[i]; i++) {
                jack_connect(client, physicalOutPorts[i], jack_port_name(input_ports[i]));
        }
    }

    if (physicalInPorts != NULL) {
        for (int i = 0; i < gNumOutChans && physicalInPorts[i]; i++) {
                jack_connect(client, jack_port_name(output_ports[i]), physicalInPorts[i]);
        }
    }

    interface->run();

    jack_deactivate(client);
    for (int i = 0; i < gNumInChans; i++) {
        jack_port_unregister(client, input_ports[i]);
    }
    for (int i = 0; i < gNumOutChans; i++) {
        jack_port_unregister(client, output_ports[i]);
    }
    jack_client_close(client);

    interface->saveState(rcfilename);

    return 0;
}

