/****************************************************************
    VocProc
    Copyright (C) 2010 Igor Brkić <igor@hyperglitch.com>

    some code taken from Faust DSP: http://faust.grame.fr/

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*****************************************************************/


#include "vproc_dsp.h"

int mydsp::getNumInputs() 	{ return 2; }

int mydsp::getNumOutputs() 	{ return 1; }

void mydsp::instanceInit(int samplingFreq) {
    fSamplingFreq = samplingFreq;

    sPitchFactor = 1.0;

    sEffect=0.0;
    cEffect=0.0;

    sOutputGain=1.0;

    cFormantVoco=0;

    sSwitch=1.0;

    cAutoTune=0.0;
    sThreshold=0.5;
    sAttack=0.0;
    sScale=1.0;
    sTranspose=1.0;

    powerIn=0;
    powerOut=0;


    fftFrameSize=2048;  // pitch detection currently doesn't work for 1024
                        // and there is aliasing present for 1024 with formant correction when
                        // large pitch shifting factor is applied
    overlap=4;

    freqOld=0;

    window=(float*)malloc(fftFrameSize*sizeof(float));
    for(int k=0;k<fftFrameSize;k++)
        window[k] = -.5*cos(M_2PI*(float)k/(float)fftFrameSize)+.5;

    gInFIFO=(float*)malloc(MAX_FRAME_LENGTH*sizeof(float)); memset(gInFIFO, 0, MAX_FRAME_LENGTH*sizeof(float));
    gIn2FIFO=(float*)malloc(MAX_FRAME_LENGTH*sizeof(float)); memset(gIn2FIFO, 0, MAX_FRAME_LENGTH*sizeof(float));
    gOutFIFO=(float*)malloc(MAX_FRAME_LENGTH*sizeof(float)); memset(gOutFIFO, 0, MAX_FRAME_LENGTH*sizeof(float));
    gOutputAccum=(float*)malloc(2*MAX_FRAME_LENGTH*sizeof(float)); memset(gOutputAccum, 0, MAX_FRAME_LENGTH*sizeof(float));


    // FFTW stuff
    fftTmpR=(double*)fftw_malloc(sizeof(double)*fftFrameSize);
    fftTmpC=(fftw_complex*)fftw_malloc(sizeof(fftw_complex)*fftFrameSize);
    fftOldC=(fftw_complex*)fftw_malloc(sizeof(fftw_complex)*fftFrameSize);
    fftCeps=(fftw_complex*)fftw_malloc(sizeof(fftw_complex)*fftFrameSize);

}

mydsp::~mydsp(){
    free(gInFIFO);
    free(gOutFIFO);
    free(gOutputAccum);
}

void mydsp::init(int samplingFreq, float **scales, int scalesNum) {
    this->scales=scales;
    this->scalesNum=scalesNum;
    instanceInit(samplingFreq);
}

void mydsp::buildUserInterface(UI* interface) {
    interface->openHorizontalBox((char*)"VocProc");
        interface->addVerticalBargraph((char*)"", &powerIn, 0.0f, 1.0f);

        interface->openVerticalBox((char*)"");

            interface->openHorizontalBox((char*)"pitch shifter");
                interface->addVerticalSlider((char*)"pitch factor", &sPitchFactor, 0.0, 0.5, 2.0, 0.01, true);
                interface->openVerticalBox((char*)"randomize/whisperize", true);
                    interface->addCheckButton((char*)"o&n/off", &cEffect);
                    interface->addVerticalSlider((char*)"", &sEffect, 0.0, 0.0, 1.0, 0.01);
                interface->closeBox();
            interface->closeBox();

            interface->openVerticalBox((char*)"formant correction / vocoder");
                interface->addCheckButton((char*)"on/o&ff", &cFormantVoco);
                interface->openHorizontalBox((char*)"");
                    interface->addLabel((char*)"       formant correction  ", NULL);
                    interface->addHorizontalSlider((char*)"", &sSwitch, 0.0, 0.0, 1.0, 1.0);
                    interface->addLabel((char*)"  vocoder       ", NULL);
                interface->closeBox();
            interface->closeBox();

            interface->openVerticalBox((char*)"automatic pitch correction");
                interface->addCheckButton((char*)"&on/off", &cAutoTune);
                interface->openHorizontalBox((char*)"");
                    interface->addVerticalSlider((char*)"threshold", &sThreshold, 0.0, 0.0, 0.07, 0.001, true);
                    interface->addVerticalSlider((char*)"attack", &sAttack, 0.0, 0.0, 1.0, 0.1, true);
                    interface->addVerticalSlider((char*)"scale", &sScale, 1.0, 0.0, (float)scalesNum-1, 1.0, true);
                    interface->addVerticalSlider((char*)"transpose", &sTranspose, 1.0, -5.0, 5.0, 1.0, true);
                interface->closeBox();
            interface->closeBox();

        interface->closeBox();
        interface->addVerticalSlider((char*)"gain", &sOutputGain, 1.0, 0.0, 1.5, 0.01);
        interface->addVerticalBargraph((char*)"", &powerOut, 0.0f, 1.0f);
    interface->closeBox();
}

