/****************************************************************

    VocProc
    Copyright (C) 2010 Igor Brkić <igor@hyperglitch.com>

    some code taken from Faust DSP: http://faust.grame.fr/

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*****************************************************************/

#ifndef VPROC_DSP_H
#define VPROC_DSP_H

#include <fftw3.h>

#include "faustqt.h"

#define M_2PI 6.2831853071795
#define MAX_FRAME_LENGTH 8192


class dsp {
 protected:
        int fSamplingFreq;

 public:
        dsp() {}
        virtual ~dsp() {}

        virtual int getNumInputs()                          = 0;
        virtual int getNumOutputs()                         = 0;
        virtual void buildUserInterface(UI* interface)      = 0;
        virtual void init(int samplingRate, float **scales, int scalesNum)= 0;
        virtual void compute(int len, float** inputs, float** outputs) 	= 0;
};


class mydsp : public dsp {
  private:
        float 	sPitchFactor, sEffect, sOutputGain;
        float   cFormantVoco, cEffect;
        float   powerIn, powerOut;
        float   sSwitch;

        float   sAttack, sScale, sThreshold, cAutoTune, sTranspose;

        float   *workspace, *gInFIFO, *gIn2FIFO, *gOutFIFO, *gOutputAccum;
        float   *window;

        long    fftFrameSize, overlap;

        float   **scales;
        int     scalesNum;

        float   freqOld;

        double  *fftTmpR;
        fftw_complex *fftTmpC;
        fftw_complex *fftOldC;
        fftw_complex *fftCeps;
        fftw_plan fftPlan;

        void    spectralEnvelope(float *env, fftw_complex *fft, int count);

        float   pitchFrequency(fftw_complex *block);
        void    setPitchFactor(float freq);

        void    phaseVocAnalysis(fftw_complex *block, float *gLastPhase, double freqPerBin, double expct, float *gAnaMagn, float *gAnaFreq);
        void    phaseVocSynthesis(fftw_complex *block, float *gSumPhase, float *gSynMagn, float *gSynFreq, double freqPerBin, double expct);

  public:
        ~mydsp();

        virtual int     getNumInputs();
        virtual int     getNumOutputs();
        static void     classInit(int samplingFreq);
        virtual void    instanceInit(int samplingFreq);
        virtual void    init(int samplingFreq, float **scales, int scalesNum);
        virtual void    buildUserInterface(UI* interface);

        virtual void    compute (int count, float** input, float** output);
};

#endif // VPROC_DSP_H
